@@ -21,8 +21,13 @@ import (
 	"encoding/pem"
 	"flag"
 	"fmt"
+	"net/http"
+	"os"
+	"strconv"
+	"sync/atomic"
 	"time"
 
+	"github.com/prometheus/client_golang/prometheus"
 	certificatesv1beta1 "k8s.io/api/certificates/v1beta1"
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -43,8 +48,10 @@ import (
 )
 
 const (
-	kubeletCAConfigMap  = "csr-controller-ca"
+	// defaultMetricsPort is the default port to expose metrics.
+	defaultMetricsPort  = 9191
 	configNamespace     = "openshift-config-managed"
+	kubeletCAConfigMap  = "csr-controller-ca"
 	machineAPINamespace = "openshift-machine-api"
 )
 
@@ -147,10 +154,10 @@ func (c *Controller) handleNewCSR(key string) error {
 	if err != nil {
 		return fmt.Errorf("failed to list machines: %v", err)
 	}
-
 	maxPending := getMaxPending(machines.Items)
+	atomic.StoreUint32(&maxPendingCSRs, uint32(maxPending))
 	if pending := recentlyPendingCSRs(c.indexer); pending > maxPending {
-		klog.Errorf("Pending CSRs: %d; Max pending allowed: %d. Difference between pending CSRs and machines > %v. Ignoring all CSRs as too many recent pending CSRs seen", pending, maxPending, maxPendingCSRs)
+		klog.Errorf("Pending CSRs: %d; Max pending allowed: %d. Difference between pending CSRs and machines > %v. Ignoring all CSRs as too many recent pending CSRs seen", pending, maxPending, maxDiffBetweenPendingCSRsAndMachinesCount)
 		return nil
 	}
 
@@ -192,7 +199,7 @@ func (c *Controller) handleNewCSR(key string) error {
 }
 
 func getMaxPending(machines []v1beta1.Machine) int {
-	return len(machines) + maxPendingCSRs
+	return len(machines) + maxDiffBetweenPendingCSRsAndMachinesCount
 }
 
 // handleErr checks if an error happened and makes sure we will retry later.
@@ -303,12 +310,40 @@ func main() {
 	// Now let's start the controller
 	stop := make(chan struct{})
 	defer close(stop)
+	startMetricsCollectionAndServer(indexer)
 	go controller.Run(1, stop)
 
 	// Wait forever
 	select {}
 }
 
+func startMetricsCollectionAndServer(indexer cache.Indexer) {
+	metricsCollector := NewMetricsCollector(indexer)
+	prometheus.MustRegister(metricsCollector)
+	metricsPort := defaultMetricsPort
+	if port, ok := os.LookupEnv("METRICS_PORT"); ok {
+		v, err := strconv.Atoi(port)
+		if err != nil {
+			klog.Fatalf("Error parsing METRICS_PORT (%q) environment variable: %v", port, err)
+		}
+		metricsPort = v
+	}
+	klog.V(4).Info("Starting server to serve prometheus metrics")
+	go startHTTPMetricServer(fmt.Sprintf("127.0.0.1:%d", metricsPort))
+}
+
+func startHTTPMetricServer(metricsPort string) {
+	mux := http.NewServeMux()
+	//TODO(vikasc): Use promhttp package for handler. This is Deprecated
+	mux.Handle("/metrics", prometheus.Handler())
+
+	server := &http.Server{
+		Addr:    metricsPort,
+		Handler: mux,
+	}
+	klog.Fatal(server.ListenAndServe())
+}
+
 // parseCSR extracts the CSR from the API object and decodes it.
 func parseCSR(obj *certificatesv1beta1.CertificateSigningRequest) (*x509.CertificateRequest, error) {
 	// extract PEM from request object
